//transforms
float4x4 tW: WORLD;      //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

          //Light Direction ( in view space !! )
float3 cam <string uiname="Cam Position";>  = {0,0,0};
float4 cAmb : COLOR <String uiname="Color";>  = {1, 1, 1, 1};
float Start = -0.2;
float Width = 1.0;
float Power = 1.0;
float Alpha = 1.0;

//texture
texture Tex <string uiname="Texture";>;
float4x4 tTex <string uiname="Texture Transform";>;                  //Texture Transform
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //set the sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tColor  <string uiname="Color Transform";>;

// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
    float4 col : COLOR0;
};

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;
    
     float dist = distance(cam, mul(Pos,tW))/200;
     
     if ((dist > Start) && (dist < Start+Width))
             dist = sin((dist-Start)/Width*3.14159);//pow( (dist - Start - (width/2)) ,2);
     else dist=0;
     
     Out.col = dist;
    
    //transform position
    Pos = mul(Pos, tWVP);
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}

// -------------------------------------------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float4 PS_Texture(float2 TexC: TEXCOORD0, float4 dist: COLOR0): COLOR
{
    float4 coltex = (tex2D(Samp, TexC));
    coltex.rgb *= pow(dist,Power);
    coltex.a *= Alpha;
    return coltex;
}

float4 PS_Color(float2 TexC: TEXCOORD0, float4 dist: COLOR0): COLOR
{
    float4 col = cAmb;
    col.rgb *= pow(dist,Power);
    col.a *= Alpha;
    return col;
}

float4 PS_TextureColor(float2 TexC: TEXCOORD0, float4 dist: COLOR0): COLOR
{
    float4 coltex = (tex2D(Samp, TexC)) * cAmb;
    coltex.rgb *= pow(dist,Power);
    coltex.a *= Alpha;
    return coltex;
}

// -------------------------------------------------------------------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------------------------------------------------------------------

technique TColorOnly
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Color();
    }
}


technique TTextureOnly
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Texture();
    }
}

technique TTextureColor
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_TextureColor();
    }
}
